@@ -27,7 +27,6 @@
 #include "sync_database_lib.h"
 #include "sync-httpd_backup.h"
 #include "sync-httpd_config.h"
-#include "sync-httpd_terms.h"
 
 /**
  * Backlog for listen operation on unix-domain sockets.
@@ -90,6 +89,37 @@ static struct MHD_Daemon *mhd;
  */
 struct SYNC_DatabasePlugin *db;
 
+/**
+ * Username and password to use for client authentication
+ * (optional).
+ */
+static char *userpass;
+
+/**
+ * Type of the client's TLS certificate (optional).
+ */
+static char *certtype;
+
+/**
+ * File with the client's TLS certificate (optional).
+ */
+static char *certfile;
+
+/**
+ * File with the client's TLS private key (optional).
+ */
+static char *keyfile;
+
+/**
+ * This value goes in the Authorization:-header.
+ */
+static char *apikey;
+
+/**
+ * Passphrase to decrypt client's TLS private key file (optional).
+ */
+static char *keypass;
+
 
 /**
  * A client has requested the given url using the given method
@@ -150,17 +180,9 @@ url_handler (void *cls,
       &SH_handler_config, MHD_HTTP_FOUND },
     { "/config", MHD_HTTP_METHOD_GET, "text/json",
       NULL, 0,
-      &SH_handler_terms, MHD_HTTP_OK },
-    { "/terms", MHD_HTTP_METHOD_GET, "text/plain",
-      NULL, 0,
-      &SH_handler_terms, MHD_HTTP_OK },
+      &SH_handler_config, MHD_HTTP_OK },
     {NULL, NULL, NULL, NULL, 0, 0 }
   };
-  static struct SH_RequestHandler h400 = {
-    "", NULL, "text/plain",
-    "Invalid account key", 0,
-    &SH_MHD_handler_static_response, MHD_HTTP_BAD_REQUEST
-  };
   static struct SH_RequestHandler h404 = {
     "", NULL, "text/html",
     "<html><title>404: not found</title></html>", 0,
@@ -219,11 +241,11 @@ url_handler (void *cls,
                                                     strlen (ac),
                                                     &account_pub.eddsa_pub))
     {
-      return SH_MHD_handler_static_response (&h400,
-                                             connection,
-                                             con_cls,
-                                             upload_data,
-                                             upload_data_size);
+      GNUNET_break_op (0);
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                         ac);
     }
     if (0 == strcasecmp (method,
                          MHD_HTTP_METHOD_OPTIONS))
@@ -299,8 +321,7 @@ url_handler (void *cls,
 
 
 /**
- * Shutdown task (magically invoked when the application is being
- * quit)
+ * Shutdown task. Invoked when the application is being terminated.
  *
  * @param cls NULL
  */
@@ -574,7 +595,39 @@ run (void *cls,
   SH_ctx = GNUNET_CURL_init (&GNUNET_CURL_gnunet_scheduler_reschedule,
                              &rc);
   rc = GNUNET_CURL_gnunet_rc_create (SH_ctx);
+  if (NULL != userpass)
+    GNUNET_CURL_set_userpass (SH_ctx,
+                              userpass);
+  if (NULL != keyfile)
+    GNUNET_CURL_set_tlscert (SH_ctx,
+                             certtype,
+                             certfile,
+                             keyfile,
+                             keypass);
 
+  // Case where APIKEY is needed.
+  if (GNUNET_OK ==
+      GNUNET_CONFIGURATION_get_value_string (config,
+                                             "sync",
+                                             "APIKEY",
+                                             &apikey))
+  {
+    char *auth_header;
+
+    GNUNET_asprintf (&auth_header,
+                     "%s: %s",
+                     MHD_HTTP_HEADER_AUTHORIZATION,
+                     apikey);
+    if (GNUNET_OK !=
+        GNUNET_CURL_append_header (SH_ctx,
+                                   auth_header))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Failed to set %s header, trying without\n",
+                  MHD_HTTP_HEADER_AUTHORIZATION);
+    }
+    GNUNET_free (auth_header);
+  }
 
   if (NULL ==
       (db = SYNC_DB_plugin_load (config)))
@@ -626,10 +679,30 @@ main (int argc,
       char *const *argv)
 {
   struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_string ('A',
+                                 "auth",
+                                 "USERNAME:PASSWORD",
+                                 "use the given USERNAME and PASSWORD for client authentication",
+                                 &userpass),
     GNUNET_GETOPT_option_flag ('C',
                                "connection-close",
                                "force HTTP connections to be closed after each request",
                                &SH_sync_connection_close),
+    GNUNET_GETOPT_option_string ('k',
+                                 "key",
+                                 "KEYFILE",
+                                 "file with the private TLS key for TLS client authentication",
+                                 &keyfile),
+    GNUNET_GETOPT_option_string ('p',
+                                 "pass",
+                                 "KEYFILEPASSPHRASE",
+                                 "passphrase needed to decrypt the TLS client private key file",
+                                 &keypass),
+    GNUNET_GETOPT_option_string ('t',
+                                 "type",
+                                 "CERTTYPE",
+                                 "type of the TLS client certificate, defaults to PEM if not specified",
+                                 &certtype),
     GNUNET_GETOPT_OPTION_END
   };
 
