@@ -144,6 +144,10 @@ struct BackupContext
    */
   unsigned int response_code;
 
+  /**
+   * Do not look for an existing order, force a fresh order to be created.
+   */
+  bool force_fresh_order;
 };
 
 
@@ -235,6 +239,7 @@ make_payment_request (const char *order_id,
     char *hdr;
     char *pfx;
     char *hn;
+    struct GNUNET_Buffer hdr_buf = { 0 };
 
     if (0 == strncasecmp ("https://",
                           SH_backend_url,
@@ -262,35 +267,19 @@ make_payment_request (const char *order_id,
       MHD_destroy_response (resp);
       return NULL;
     }
-    // FIXME: support instances?
+
+    GNUNET_buffer_write_str (&hdr_buf, pfx);
+    GNUNET_buffer_write_str (&hdr_buf, "pay/");
+    GNUNET_buffer_write_str (&hdr_buf, hn);
+    GNUNET_buffer_write_path (&hdr_buf, order_id);
+    /* No session ID */
+    GNUNET_buffer_write_path (&hdr_buf, "");
     if (NULL != token)
     {
-      char tok[256];
-
-      /* This path should not be taken, as we disabled tokens */
-      GNUNET_assert (NULL !=
-                     GNUNET_STRINGS_data_to_string (token,
-                                                    sizeof (*token),
-                                                    tok,
-                                                    sizeof (tok)));
-      GNUNET_asprintf (&hdr,
-                       "%spay/%s%s%s/?c=%s",
-                       pfx,
-                       hn,
-                       ('/' == hn[strlen (hn) - 1])
-                       ? ""
-                       : "/",
-                       order_id,
-                       tok);
-    }
-    else
-    {
-      GNUNET_asprintf (&hdr,
-                       "%spay/%s/%s/",
-                       pfx,
-                       hn,
-                       order_id);
+      GNUNET_buffer_write_str (&hdr_buf, "?c=");
+      GNUNET_buffer_write_data_encoded (&hdr_buf, token, sizeof (*token));
     }
+    hdr = GNUNET_buffer_reap_str (&hdr_buf);
     GNUNET_break (MHD_YES ==
                   MHD_add_response_header (resp,
                                            "Taler",
@@ -330,21 +319,17 @@ proposal_cb (void *cls,
                 "Backend returned status %u/%u\n",
                 por->hr.http_status,
                 (unsigned int) por->hr.ec);
-    GNUNET_break (0);
-    bc->resp = TALER_MHD_make_json_pack ("{s:I, s:s, s:I, s:I, s:O}",
-                                         "code",
-                                         (json_int_t)
-                                         TALER_EC_SYNC_PAYMENT_CREATE_BACKEND_ERROR,
-                                         "hint",
-                                         "Failed to setup order with merchant backend",
-                                         "backend-ec",
-                                         (json_int_t) por->hr.ec,
-                                         "backend-http-status",
-                                         (json_int_t) por->hr.http_status,
-                                         "backend-reply",
-                                         por->hr.reply);
-    GNUNET_assert (NULL != bc->resp);
-    bc->response_code = MHD_HTTP_FAILED_DEPENDENCY;
+    GNUNET_break_op (0);
+    bc->resp = TALER_MHD_MAKE_JSON_PACK (
+      TALER_JSON_pack_ec (TALER_EC_SYNC_PAYMENT_CREATE_BACKEND_ERROR),
+      GNUNET_JSON_pack_uint64 ("backend-ec",
+                               por->hr.ec),
+      GNUNET_JSON_pack_uint64 ("backend-http-status",
+                               por->hr.http_status),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_object_incref ("backend-reply",
+                                        (json_t *) por->hr.reply)));
+    bc->response_code = MHD_HTTP_BAD_GATEWAY;
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -524,35 +509,39 @@ begin_payment (struct BackupContext *bc,
                int pay_req)
 {
   json_t *order;
-  enum GNUNET_DB_QueryStatus qs;
 
-  qs = db->lookup_pending_payments_by_account_TR (db->cls,
-                                                  &bc->account,
-                                                  &ongoing_payment_cb,
-                                                  bc);
-  if (qs < 0)
+  if (! bc->force_fresh_order)
   {
-    struct MHD_Response *resp;
-    MHD_RESULT ret;
+    enum GNUNET_DB_QueryStatus qs;
 
-    resp = TALER_MHD_make_error (TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                 "pending payments");
-    ret = MHD_queue_response (bc->con,
-                              MHD_HTTP_INTERNAL_SERVER_ERROR,
-                              resp);
-    GNUNET_break (MHD_YES == ret);
-    MHD_destroy_response (resp);
-    return ret;
-  }
-  if (NULL != bc->existing_order_id)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "Have existing order, waiting for `%s' to complete\n",
-                bc->existing_order_id);
-    await_payment (bc,
-                   GNUNET_TIME_UNIT_ZERO /* no long polling */,
-                   bc->existing_order_id);
-    return MHD_YES;
+    qs = db->lookup_pending_payments_by_account_TR (db->cls,
+                                                    &bc->account,
+                                                    &ongoing_payment_cb,
+                                                    bc);
+    if (qs < 0)
+    {
+      struct MHD_Response *resp;
+      MHD_RESULT ret;
+
+      resp = TALER_MHD_make_error (TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                   "pending payments");
+      ret = MHD_queue_response (bc->con,
+                                MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                resp);
+      GNUNET_break (MHD_YES == ret);
+      MHD_destroy_response (resp);
+      return ret;
+    }
+    if (NULL != bc->existing_order_id)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Have existing order, waiting for `%s' to complete\n",
+                  bc->existing_order_id);
+      await_payment (bc,
+                     GNUNET_TIME_UNIT_ZERO /* no long polling */,
+                     bc->existing_order_id);
+      return MHD_YES;
+    }
   }
   GNUNET_CONTAINER_DLL_insert (bc_head,
                                bc_tail,
@@ -561,10 +550,13 @@ begin_payment (struct BackupContext *bc,
               "Suspending connection while creating order at `%s'\n",
               SH_backend_url);
   MHD_suspend_connection (bc->con);
-  order = json_pack ("{s:o, s:s, s:s}",
-                     "amount", TALER_JSON_from_amount (&SH_annual_fee),
-                     "summary", "annual fee for sync service",
-                     "fulfillment_url", SH_fulfillment_url);
+  order = GNUNET_JSON_PACK (
+    TALER_JSON_pack_amount ("amount",
+                            &SH_annual_fee),
+    GNUNET_JSON_pack_string ("summary",
+                             "annual fee for sync service"),
+    GNUNET_JSON_pack_string ("fulfillment_url",
+                             SH_fulfillment_url));
   bc->po = TALER_MERCHANT_orders_post2 (SH_ctx,
                                         SH_backend_url,
                                         order,
@@ -685,6 +677,15 @@ SH_backup_post (struct MHD_Connection *connection,
     bc->hc.cc = &cleanup_ctx;
     bc->con = connection;
     bc->account = *account;
+    {
+      const char *fresh;
+
+      fresh = MHD_lookup_connection_value (connection,
+                                           MHD_GET_ARGUMENT_KIND,
+                                           "fresh");
+      if (NULL != fresh)
+        bc->force_fresh_order = true;
+    }
     *con_cls = bc;
 
     /* now setup 'bc' */
